float4x4 World;
float4x4 View;
float4x4 Projection;

// light
float3 LightDirection;
float4 EyePosition;
float4 AmbientLightColor;
float4 DiffuseLightColor;
float4 SpecularLightColor;

// texture
texture DiffuseTexture;
sampler2D DiffuseSampler = sampler_state {
    Texture = <DiffuseTexture>;
    MinFilter = LINEAR;
    MipFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = WRAP;
    AddressV = MIRROR;
};

struct VertexShaderInput
{
    float4 Position : POSITION0;
    float4 DiffuseTexCoord : TEXCOORD0;
};

struct VertexShaderOutput
{
    float4 Position : POSITION;	// transformed position
    float3 DiffuseTexCoord : TEXCOORD0;		// texture
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
	output.DiffuseTexCoord = input.DiffuseTexCoord;

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
    // TODO: add your pixel shader code here.

    return tex2D(DiffuseSampler, input.DiffuseTexCoord);
}

technique DefaultTextureTechnique
{
    pass Pass1
    {
        // TODO: set renderstates here.

        VertexShader = compile vs_2_0 VertexShaderFunction();
        PixelShader = compile ps_2_0 PixelShaderFunction();
    }
}

//////////////////////////////////////////////////////
// per-pixel lighting 
struct PerPixelLightingVertexShaderInput 
{
    float4 Position : POSITION;
    float4 DiffuseTexCoord : TEXCOORD0;
    float4 Normal : TEXCOORD1;
};

struct PerPixelLightingVertexShaderOutput 
{
    float4 Position : POSITION;
    float4 DiffuseTexCoord : TEXCOORD0;
    float4 Normal : TEXCOORD1;
};

struct PerPixelLightingPixelShaderInput
{
	float4 DiffuseTexCoord : TEXCOORD0;
	float4 Normal: TEXCOORD1;
};

/*
PerPixelLightingVertexShaderOutput PerPixelLightingVertexShaderFunction(PerPixelLightingVertexShaderInput input)
{
    PerPixelLightingVertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
	output.DiffuseTexCoord = input.DiffuseTexCoord;
	output.Normal = mul(input.Normal, World);

    return output;
}
*/

PerPixelLightingVertexShaderOutput PerPixelLightingVertexShaderFunction(
	float3 Position : POSITION,
	float3 TexCoord : TEXCOORD0,
    float3 Normal : NORMAL)
{
    PerPixelLightingVertexShaderOutput output;

    float4 worldPosition = mul(float4(Position, 1.0), World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
	output.DiffuseTexCoord = float4(TexCoord, 0.0);
	output.Normal = mul(float4(Normal, 0.0), World);

    return output;
}

float4 PerPixelLightingPixelShaderFunction(PerPixelLightingPixelShaderInput input) : COLOR
{
    // TODO: add your pixel shader code here.
	float4 directionToLight = float4(normalize(LightDirection), 0.0);
	float diffuseIntensity =  saturate(dot(directionToLight, input.Normal));
	float4 diffuse = DiffuseLightColor * diffuseIntensity;
	float4 lightColor = diffuse + AmbientLightColor;
	lightColor.w = 1.0;
	
    return lightColor * tex2D(DiffuseSampler, input.DiffuseTexCoord);
}

technique PerPixelDiffuseLightingTextureTechnique
{
    pass Pass1
    {
        // TODO: set renderstates here.

        VertexShader = compile vs_2_0 PerPixelLightingVertexShaderFunction();
        PixelShader = compile ps_2_0 PerPixelLightingPixelShaderFunction();
    }
}
